/**
 * PixelMatrix v1.0.0
 *
 * URL: http://blog.wantedlink.de
 * (c) 2015 Mario Wolf
 */

"use strict";

// http://paulirish.com/2011/requestanimationframe-for-smart-animating
window.requestAnimFrame = (function(){
	return  window.requestAnimationFrame       || 
          	window.webkitRequestAnimationFrame || 
          	window.mozRequestAnimationFrame    || 
          	window.oRequestAnimationFrame      || 
          	window.msRequestAnimationFrame     || 
          	function( callback ){
           		window.setTimeout(callback, 1000/60);
          	};
})();

window.cancelRequestAnimFrame = (function(){
	return 	window.cancelAnimationFrame          		||
			window.webkitCancelRequestAnimationFrame    ||
			window.mozCancelRequestAnimationFrame       ||
			window.oCancelRequestAnimationFrame     	||
			window.msCancelRequestAnimationFrame        ||
			clearTimeout
})();

/**
 * mwPixelMatrix root namespace.
 * 
 * @namespace mwPixelMatrix
 */
 var mwPixelMatrix = mwPixelMatrix || {};

/**
 * Core PixelMatrix
 * 
 * @namespace mwPixelMatrix
 */
mwPixelMatrix.Core = function()
{

	var t = this;
	t.canvasID = "mat";
	t.field = null;
	t.canvas = null;
	t.ctx = null;

	t.boxes = [];

	/*
	 * constructor
	 */
	var __construct = function() {

		t.canvas = document.getElementById(t.canvasID);
		t.canvas.width = 500;
		t.canvas.height = 300;
		t.ctx = t.canvas.getContext('2d');
		t.ctx.lineJoin = t.ctx.lineCap = 'round';	//set round corners

		animate();	//call loop func
	}();

	/*
	 * loop
	 */
	function animate() {
		t.ctx.save();
		t.ctx.clearRect(0, 0, t.canvas.width, t.canvas.height);
		draw();
		t.ctx.restore();
		requestAnimFrame( animate );
	}

	/*
	 * draw all objects
	 */
	function draw(){
		addBox();

		// loop over boxes and draw 
		for(var i = 0; i < t.boxes.length; i++){ 

			t.boxes[i].y += t.boxes[i].speed * 0.5;
			t.ctx.fillStyle = t.boxes[i].fillStyle;
			t.ctx.fillRect(t.boxes[i].x, t.boxes[i].y, t.boxes[i].width, t.boxes[i].height);
		}

		// remove box when height arrive
		if(t.boxes[0].y > t.canvas.height){ 
			t.boxes.splice(0,1); 	// splice(index, count)
		}	
	}

	/*
	 * set random color and fill array with objects
	 */
	function addBox(){

		var rndColorHex = '#' + randomHexColor() + randomHexColor() + randomHexColor();
		var rndX = randomInt(0, t.canvas.width-10); // random x position - box size
		var rndSpeed = randomInt(1,3);

		t.boxes.push(new mwPixelMatrix.Box(rndX, -10, 10, 10, rndSpeed, rndColorHex)); // cread and add boxs to array
	}


	/*
	 * randomInt generate random number in a range
	 *
	 * @param min {int} minimum
	 * @param max {int} maximum
	 */
	function randomInt(min, max) {
		return Math.floor(Math.random() * (max - min + 1)) + min;
	}

	/*
	 * randomHexColor generate random color hex
	 */
	function randomHexColor(){

		var hex = Math.floor((Math.random() * 255)).toString(16);

		if(hex.length < 2) {
	        hex = "0" + hex;
	    }

		return hex.toUpperCase();
	}
};

/**
 * Box Class
 * 
 * @namespace mwPixelMatrix
 * @class mwPixelMatrix.Box
 *
 * @param l_x {int} position X-coordinate
 * @param l_y {int} position Y-coordinate
 * @param l_width {int} width in px
 * @param l_height {int} height in px
 * @param l_speed {int} speed
 * @param l_color {string} colorname
 */
mwPixelMatrix.Box = function(l_x, l_y, l_width, l_height, l_speed, l_color)
{
	var t = this;
 	t.x = l_x;
 	t.y = l_y;
 	t.width = l_width;
 	t.height = l_height;
 	t.speed = l_speed;
 	t.fillStyle = l_color;
};
